@@ -14,16 +14,14 @@ from snakefmt.parser.parser import Parser, comment_start
 from snakefmt.parser.syntax import (
     COMMENT_SPACING,
     TAB,
+    InlineSingleParam,
     Parameter,
     ParameterSyntax,
-    RuleInlineSingleParam,
     SingleParam,
     Syntax,
 )
 from snakefmt.types import TokenIterator
 
-rule_like_formatted = {"rule", "checkpoint"}
-
 triple_quote_matcher = re.compile(
     r"^\s*(\w?\"{3}.*?\"{3})|^\s*(\w?'{3}.*?'{3})", re.DOTALL | re.MULTILINE
 )
@@ -96,8 +94,9 @@ class Formatter(Parser):
                 formatted = "".join(formatted_lines[:-1])  # Remove the 'pass' line
             else:
                 formatted = self.run_black_format_str(self.buffer, self.target_indent)
-            code_indent = self.context.code_indent
-            formatted = textwrap.indent(formatted, f"{TAB * code_indent}")
+            code_indent = self.syntax.code_indent
+            if code_indent is not None:
+                formatted = textwrap.indent(formatted, f"{TAB * code_indent}")
 
         # Re-add newline removed by black for proper parsing of comments
         if self.buffer.endswith("\n\n"):
@@ -111,13 +110,14 @@ class Formatter(Parser):
         self.buffer = ""
 
     def process_keyword_context(self, in_global_context: bool):
-        cur_indent = self.context.cur_indent
+        cur_indent = self.syntax.cur_indent
         self.add_newlines(cur_indent, in_global_context=in_global_context)
-        formatted = (
-            f"{TAB * cur_indent}{self.context.keyword_name}:{self.context.comment}\n"
-        )
+        formatted = f"{TAB * cur_indent}{self.syntax.keyword_line}"
+        if self.syntax.enter_context:
+            formatted += ":"
+        formatted += f"{self.syntax.comment}\n"
         self.result += formatted
-        self.last_recognised_keyword = self.context.keyword_name
+        self.last_recognised_keyword = self.syntax.keyword_name
 
     def process_keyword_param(
         self, param_context: ParameterSyntax, in_global_context: bool
@@ -222,10 +222,9 @@ class Formatter(Parser):
 
         p_class = parameters.__class__
         single_param = issubclass(p_class, SingleParam)
-        inline_fmting = single_param
-        # Cancel single param formatting if in rule-like context and param not inline
-        if in_rule and p_class is not RuleInlineSingleParam:
-            inline_fmting = False
+        inline_fmting = False
+        if p_class is InlineSingleParam:
+            inline_fmting = True
 
         result = f"{used_indent}{parameters.keyword_name}:"
         if inline_fmting:
